#!/usr/bin/env python
import os
from pathlib import Path
from typing import List, Optional, Sequence

from ltpylib import files, filters

FIND_REPOS_RECURSION_EXCLUDES = frozenset([
  'node_modules',
])


def is_file_part_of_git_repo(file_path: Path) -> bool:
  return git_repo_root_for_file(file_path) is not None


def git_repo_root_for_file(file_path: Path) -> Optional[Path]:
  if file_path.is_dir() and file_path.joinpath(".git").is_dir():
    return file_path

  for parent_file in file_path.parents:
    if parent_file.joinpath(".git").is_dir():
      return parent_file

  return None


def resolve_file_relative_to_git_base_dir(file_path: Path, current_dir: Path = Path(os.getcwd())) -> Optional[Path]:
  git_repo_root = git_repo_root_for_file(current_dir)
  if not git_repo_root:
    return None

  maybe_file = git_repo_root.joinpath(file_path)
  return maybe_file if maybe_file.exists() else None


def filter_invalid_repos(git_repos: List[Path]) -> List[Path]:
  filtered = []
  for repo in git_repos:
    if not repo.is_dir():
      continue

    if not repo.joinpath('.git').is_dir():
      continue

    filtered.append(repo)

  return filtered


def print_repos(git_repos: List[Path]):
  for repo in git_repos:
    print(repo.as_posix())


def find_git_repos(
  base_dir: Path,
  max_depth: int = -1,
  recursion_include_patterns: Sequence[str] = None,
  recursion_exclude_patterns: Sequence[str] = None,
  recursion_includes: Sequence[str] = None,
  recursion_excludes: Sequence[str] = FIND_REPOS_RECURSION_EXCLUDES
) -> List[Path]:
  dotgit_dirs = files.find_children(
    base_dir,
    break_after_match=True,
    include_files=False,
    max_depth=max_depth,
    includes=['.git'],
    recursion_include_patterns=recursion_include_patterns,
    recursion_exclude_patterns=recursion_exclude_patterns,
    recursion_includes=recursion_includes,
    recursion_excludes=recursion_excludes
  )
  return [dotgit.parent for dotgit in dotgit_dirs]


def add_git_dirs(
  git_repos: List[Path],
  add_dir: List[Path],
  include_patterns: List[str] = None,
  exclude_patterns: List[str] = None,
  max_depth: int = -1,
  recursion_include_patterns: Sequence[str] = None,
  recursion_exclude_patterns: Sequence[str] = None,
  recursion_includes: Sequence[str] = None,
  recursion_excludes: Sequence[str] = FIND_REPOS_RECURSION_EXCLUDES
) -> List[Path]:
  for git_dir in add_dir:
    if not git_dir.is_dir():
      continue

    add_dir_repos = find_git_repos(
      git_dir,
      max_depth=max_depth,
      recursion_include_patterns=recursion_include_patterns,
      recursion_exclude_patterns=recursion_exclude_patterns,
      recursion_includes=recursion_includes,
      recursion_excludes=recursion_excludes
    )
    add_dir_repos.sort()
    for git_repo in add_dir_repos:
      if filters.should_skip(git_repo, exclude_patterns=exclude_patterns, include_patterns=include_patterns):
        continue

      if git_repo in git_repos:
        continue

      git_repos.append(git_repo)

  return git_repos



#!/usr/bin/env python

import logging
import subprocess
from pathlib import Path
from typing import List, Union

from ltpylib import files, procs

FILE_EXT_MAPPINGS = {
  "yml": "yaml",
}


def prettify(
  files_to_prettify: Union[Path, List[Path]],
  file_type: str = None,
  compact: bool = False,
  debug_mode: bool = False,
  verbose: bool = False,
):
  if not isinstance(files_to_prettify, list):
    files_to_prettify = [files_to_prettify]

  for file in files_to_prettify:
    if file_type:
      single_file_type = file_type
    else:
      single_file_type = FILE_EXT_MAPPINGS.get(file.suffix[1:], file.suffix[1:])

    func_for_type = globals()["prettify_" + single_file_type + "_file"]
    if not callable(func_for_type):
      raise ValueError("Unsupported file type: file=%s type=%s" % (file.as_posix(), single_file_type))

    func_for_type(file, compact=compact, debug_mode=debug_mode, verbose=verbose)
    if verbose:
      logging.debug("Updated %s", file.as_posix())


def prettify_html_file(
  file: Path,
  compact: bool = False,
  debug_mode: bool = False,
  verbose: bool = False,
):
  tidy_executable = "/usr/bin/tidy" if Path("/usr/bin/tidy").exists() else "tidy"
  tidy_args = [
    tidy_executable,
    "-icm",
    "-wrap",
    "200",
    "--doctype",
    "omit",
    "--indent",
    "yes",
    "--vertical-space",
    "no",
  ]

  if not verbose:
    tidy_args.extend([
      "-quiet",
      "--show-warnings",
      "no",
    ])

  tidy_args.append(file.as_posix())

  result = procs.run(tidy_args)
  check_proc_result(file, result, should_have_stdout=False, allow_exit_codes=[0, 1])


def prettify_json_file(
  file: Path,
  compact: bool = False,
  debug_mode: bool = False,
  verbose: bool = False,
):
  jq_args = ["--sort-keys", ".", file.as_posix()]
  if compact:
    jq_args.insert(0, "--compact-output")

  result = procs.run(["jq"] + jq_args)
  check_proc_result(file, result)
  files.write_file(file, result.stdout)


def prettify_sql_file(
  file: Path,
  compact: bool = False,
  debug_mode: bool = False,
  verbose: bool = False,
):
  sql_formatter_cmd = [
    "sql-formatter",
    "--language",
    "sqlite",
    "--config",
    Path.home().joinpath(".config/sql-formatter/sqlite.json").as_posix(),
    "--output",
    file.as_posix(),
    file.as_posix(),
  ]
  procs.run_with_regular_stdout(sql_formatter_cmd, check=True)


def prettify_xml_file(
  file: Path,
  compact: bool = False,
  debug_mode: bool = False,
  verbose: bool = False,
):
  result = procs.run(["xmllint", "--format", file.as_posix()])
  check_proc_result(file, result)
  files.write_file(file, result.stdout)


def prettify_yaml_file(
  file: Path,
  compact: bool = False,
  debug_mode: bool = False,
  verbose: bool = False,
):
  result = procs.run(["yq", "--yaml-roundtrip", "--indentless-lists", "--sort-keys", ".", file.as_posix()])
  check_proc_result(file, result)
  files.write_file(file, result.stdout)


def check_proc_result(file: Path, result: subprocess.CompletedProcess, should_have_stdout: bool = True, allow_exit_codes: List[int] = None):
  exit_code = result.returncode
  proc_succeeded = exit_code == 0
  if allow_exit_codes and exit_code in allow_exit_codes:
    proc_succeeded = True

  if should_have_stdout and not result.stdout:
    raise Exception("Issue prettifying file: file=%s status=%s stderr=%s" % (file.as_posix(), exit_code, result.stderr))

  if result.stderr:
    if proc_succeeded:
      logging.warning(result.stderr)
    else:
      raise Exception("Issue prettifying file: file=%s status=%s stderr=%s stdout=%s" % (file.as_posix(), exit_code, result.stderr, result.stderr))

  if not proc_succeeded:
    result.check_returncode()



#!/usr/bin/env python
import logging
import subprocess
import sys
from pathlib import Path
from types import TracebackType
from typing import Any, Callable, IO, List, Optional, Tuple, Type, Union


class CalledProcessErrorWithOutput(subprocess.CalledProcessError):

  def __init__(self, returncode, cmd, output=None, stderr=None):
    super(CalledProcessErrorWithOutput, self).__init__(returncode, cmd, output, stderr)

  def __str__(self):
    stdout_and_stderr = ""

    if self.output:
      stdout_and_stderr += "\n\nstdout\n---\n%s" % self.output

    if self.stderr:
      stdout_and_stderr += "\n\nstderr\n---\n%s" % self.stderr

    if self.returncode and self.returncode < 0:
      try:
        import signal

        return "Command '%s' died with %r.%s" % (self.cmd, signal.Signals(-self.returncode), stdout_and_stderr)
      except ValueError:
        return "Command '%s' died with unknown signal %d.%s" % (self.cmd, -self.returncode, stdout_and_stderr)
    else:
      return "Command '%s' returned non-zero exit status %d.%s" % (self.cmd, self.returncode, stdout_and_stderr)


def run_popen(
  *popenargs,
  cwd: Union[str, bytes, Path] = None,
  shell: bool = False,
  stdout: Optional[Union[int, IO]] = subprocess.PIPE,
  stderr: Optional[Union[int, IO]] = subprocess.PIPE,
  **kwargs,
) -> subprocess.Popen:
  kwargs['universal_newlines'] = True

  return subprocess.Popen(
    *popenargs,
    cwd=cwd,
    shell=shell,
    stdout=stdout,
    stderr=stderr,
    **kwargs,
  )


def run(
  *popenargs,
  input: Union[bytes, str, None] = None,
  timeout: Optional[float] = None,
  check: bool = False,
  cwd: Union[str, bytes, Path] = None,
  shell: bool = False,
  stdout: Optional[Union[int, IO]] = subprocess.PIPE,
  stderr: Optional[Union[int, IO]] = subprocess.PIPE,
  **kwargs,
) -> subprocess.CompletedProcess:
  kwargs['universal_newlines'] = True

  result = subprocess.run(
    *popenargs,
    input=input,
    timeout=timeout,
    check=False,
    cwd=cwd,
    shell=shell,
    stdout=stdout,
    stderr=stderr,
    **kwargs,
  )

  if check:
    check_returncode_with_output(result)

  return result


def run_with_logging_output(
  *popenargs,
  input: Union[bytes, str, None] = None,
  timeout: Optional[float] = None,
  check: bool = False,
  cwd: Union[str, bytes, Path] = None,
  shell: bool = False,
  level: int = logging.INFO,
  **kwargs,
) -> subprocess.CompletedProcess:
  from ltpylib import logs

  with logs.LogPipe(level=level) as log_pipe:
    return run(
      *popenargs,
      input=input,
      timeout=timeout,
      check=check,
      cwd=cwd,
      shell=shell,
      stdout=log_pipe,
      stderr=log_pipe,
      **kwargs,
    )


def run_with_regular_stdout(
  *popenargs,
  input: Union[bytes, str, None] = None,
  timeout: Optional[float] = None,
  check: bool = False,
  cwd: Union[str, bytes, Path] = None,
  shell: bool = False,
  **kwargs,
) -> subprocess.CompletedProcess:
  return run(
    *popenargs,
    input=input,
    timeout=timeout,
    check=check,
    cwd=cwd,
    shell=shell,
    stdout=sys.stdout,
    stderr=sys.stderr,
    **kwargs,
  )


def run_and_parse_output(
  *popenargs,
  input: Union[bytes, str, None] = None,
  timeout: Optional[float] = None,
  check: bool = False,
  cwd: Union[str, bytes, Path] = None,
  shell: bool = False,
  **kwargs,
) -> Tuple[int, str]:
  kwargs['stdout'] = subprocess.PIPE

  result = run(
    *popenargs,
    input=input,
    timeout=timeout,
    check=check,
    cwd=cwd,
    shell=shell,
    **kwargs,
  )

  return result.returncode, result.stdout


def run_and_parse_output_on_success(
  *popenargs,
  input: Union[bytes, str, None] = None,
  timeout: Optional[float] = None,
  check: bool = True,
  cwd: Union[str, bytes, Path] = None,
  shell: bool = False,
  **kwargs,
) -> str:
  kwargs['stdout'] = subprocess.PIPE

  result = run(
    *popenargs,
    input=input,
    timeout=timeout,
    check=check,
    cwd=cwd,
    shell=shell,
    **kwargs,
  )

  return result.stdout


def check_returncode_with_output(result: subprocess.CompletedProcess):
  if result.returncode:
    raise CalledProcessErrorWithOutput(result.returncode, result.args, result.stdout, result.stderr)


def get_procs_from_name(name_matcher: str) -> List[Tuple[int, str]]:
  matched_procs = []
  exit_code, output = run_and_parse_output(['pgrep', '-fl', name_matcher])
  if exit_code > 0:
    return matched_procs

  for line in output.splitlines():
    parts = line.partition(' ')
    matched_procs.append((int(parts[0]), parts[2]))

  return matched_procs


def proc_debug_string(proc) -> str:
  """
  :type proc: psutil.Process
  """
  import collections
  import psutil

  info = collections.OrderedDict()
  info['pid'] = proc.pid
  try:
    info["name"] = proc.name()
    if proc._create_time:
      info['started'] = psutil._pprint_secs(proc._create_time)
  except psutil.ZombieProcess:
    info["status"] = "zombie"
  except psutil.NoSuchProcess:
    info["status"] = "terminated"
  except psutil.AccessDenied:
    pass
  info['cmdline'] = ' '.join(proc.cmdline())
  return "%s.%s(%s)" % (
    proc.__class__.__module__,
    proc.__class__.__name__,
    ", ".join(["%s=%r" % (k, v) for k, v in info.items()]),
  )


def stop_proc_by_name(name_matcher: str) -> bool:
  import psutil

  matched_procs = get_procs_from_name(name_matcher)
  if not matched_procs:
    return False

  if len(matched_procs) > 1:
    raise Exception('Multiple processes found: %s' % matched_procs)

  proc = psutil.Process(matched_procs[0][0])
  proc.terminate()
  return True


def stop_proc_by_pid(pid: Any) -> bool:
  import psutil

  proc = psutil.Process(int(pid))
  if proc.is_running():
    proc.terminate()
  else:
    return False
  return True


def await_termination(pid: int, timeout: int = 30, sleep_time: int = 1, log_level: int = 20):
  """
  :param log_level: default is logging.INFO
  :return: None
  """
  import logging
  import psutil
  from time import sleep

  if pid is None:
    return

  proc = psutil.Process(pid)
  total_time = 0
  while total_time <= timeout:
    if proc.is_running():
      logging.log(log_level, 'STATUS: loop %s seconds - process is still running, please wait...', total_time)
    else:
      break

    total_time += sleep_time
    sleep(sleep_time)

  if proc.is_running():
    logging.error('STATUS: Process still running after %s seconds, sending SIGKILL to terminate...', total_time)
    proc.kill()
    proc.wait(5)
    logging.log(log_level, 'STATUS: Process killed.')
  else:
    logging.log(log_level, 'STATUS: Process successfully shutdown after %s seconds.', total_time)


orig_unraisablehook = None


def disable_unraisablehook():
  if not hasattr(sys, "unraisablehook"):
    return

  global orig_unraisablehook
  if orig_unraisablehook is None:
    orig_unraisablehook = sys.unraisablehook

  sys.unraisablehook = ignoring_unraisablehook


def enable_unraisablehook():
  if not hasattr(sys, "unraisablehook"):
    return

  global orig_unraisablehook
  if orig_unraisablehook is None:
    raise Exception("sys.unraisablehook was never overridden")
  sys.unraisablehook = orig_unraisablehook
  orig_unraisablehook = None


def ignoring_unraisablehook(*args, **kwargs):
  pass


class catch_unraisable_exception:

  def __init__(self) -> None:
    self.unraisable: Optional["sys.UnraisableHookArgs"] = None
    self._old_hook: Optional[Callable[["sys.UnraisableHookArgs"], Any]] = None

  def _hook(self, unraisable: "sys.UnraisableHookArgs") -> None:
    # Storing unraisable.object can resurrect an object which is being
    # finalized. Storing unraisable.exc_value creates a reference cycle.
    self.unraisable = unraisable

  def __enter__(self) -> "catch_unraisable_exception":
    self._old_hook = sys.unraisablehook
    sys.unraisablehook = self._hook
    return self

  def __exit__(
    self,
    exc_type: Optional[Type[BaseException]],
    exc_val: Optional[BaseException],
    exc_tb: Optional[TracebackType],
  ) -> None:
    assert self._old_hook is not None
    sys.unraisablehook = self._old_hook
    self._old_hook = None
    del self.unraisable


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python

import logging
from typing import List


class ChromeExtensions(object):

  def __init__(self):
    self.installed: List[ChromeExtension] = []
    self.pinned_extensions: List[str] = []
    self.toolbar: List[str] = []


class ChromeExtension(object):

  def __init__(self, extension_id: str, name: str, description: str):
    self.extension_id: str = extension_id
    self.name: str = name
    self.description: str = description


def create_chrome_extension(extension_id: str) -> ChromeExtension:
  import requests
  from ltpylib import htmlparser

  url = f"https://chrome.google.com/webstore/detail/{extension_id}"
  logging.debug(f"Getting extension details: {url}")

  response = requests.get(url)
  parsed_html = htmlparser.create_parser(response.text)
  name_elem = parsed_html.select_one("meta[property='og:title']")
  description_elem = parsed_html.select_one("meta[property='og:description']")

  return ChromeExtension(
    extension_id,
    name_elem.get("content") if name_elem else None,
    description_elem.get("content") if description_elem else None,
  )



#!/usr/bin/env python
from typing import List

import requests

from ltpylib import requests_helper

DD_API_BASE = "https://api.datadoghq.com/api/v1"


class DatadogApi(object):

  def __init__(
    self,
    dd_api_key: str,
    dd_application_key: str = None,
    base_url: str = DD_API_BASE,
  ):
    self._dd_api_key: str = dd_api_key
    self._dd_application_key: str = dd_application_key

    self.base_url: str = base_url.removesuffix("/")
    self.session: requests.Session = requests.Session()
    self.session.headers.update({
      "Accept": "application/json",
      "Content-Type": "application/json",
      "DD-API-KEY": dd_api_key,
    })
    if dd_application_key:
      self.session.headers.update({"DD-APPLICATION-KEY": dd_application_key})

  def url(self, path: str) -> str:
    if not path.startswith("/"):
      path = "/" + path

    return self.base_url + path

  def get_dashboard(self, dashboard_id: str) -> dict:
    return requests_helper.maybe_throw(self.session.get(self.url(f"/dashboard/{dashboard_id}"))).json()

  def get_dashboards(self) -> List[dict]:
    return requests_helper.maybe_throw(self.session.get(self.url("/dashboard"))).json()["dashboards"]

  def update_dashboard(self, dashboard_id: str, dashboard: dict) -> dict:
    return requests_helper.maybe_throw(self.session.put(self.url(f"/dashboard/{dashboard_id}"), json=dashboard)).json()
